﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2010/10/20 初版
//        2011/07/15 ２版  実行オプションパラメータ追加
//
// GS2 Sample Program
// -----------------------------------------------------------------


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace GS2
{

    public partial class Form_GS2 : Form
    {
        // 取込クラス
        FVIL.Video.CFviVideo m_video;

        // 画像、パタン 関連インスタンス
        FVIL.Data.CFviPattern m_pattern = new FVIL.Data.CFviPattern();
        FVIL.Data.CFviImage m_image = new FVIL.Data.CFviImage();

        // グレイサーチ2 関連インスタンス
        FVIL.GS2.CFviGS2 m_gs2 = new FVIL.GS2.CFviGS2();
        FVIL.GS2.CFviGS2Pattern m_gs2ptn = new FVIL.GS2.CFviGS2Pattern();

        // 表示用オーバーレイクラス宣言
        private FVIL.GDI.CFviOverlay m_overlay = new FVIL.GDI.CFviOverlay();

        // 処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        // iniファイル設定フォーム
        Form_BROWSEINI formBroseIni = new Form_BROWSEINI();

        // ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;

        // 時間測定用
        private double msec = 0;
        
        public Form_GS2()
        {
            InitializeComponent();
        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理、使用許諾を確認します。
        /// </remarks>
        private void Form_GS2_Load(object sender, EventArgs e)
        {
            //使用許諾の表示
            dialogLicence licence_dialog = new dialogLicence();
            if (licence_dialog.ShowDialog(this) != DialogResult.OK)
            {
                this.Close();
                return;
            }

            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //変数の初期化
            m_flgVideoOpened = false;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0";

            //トリガモードコンボボックス初期化
            ComboBoxTrigger.Items.Clear();
            ComboBoxTrigger.Items.Add("ノーマル");
            ComboBoxTrigger.Items.Add("内部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ");
            ComboBoxTrigger.SelectedIndex = 0;

            //サーチパラメータ用選択コンボボックス初期化
            //精度
            comboBoxPrecision.Items.Clear();
            comboBoxPrecision.Items.Add("通常精度");
            comboBoxPrecision.Items.Add("高精度");
            comboBoxPrecision.Items.Add("超高精度");
            comboBoxPrecision.Items.Add("ウルトラ超高精度");
            comboBoxPrecision.SelectedIndex = 0;

            // 反転パタン検出
            comboBoxReverseMode.Items.Clear();
            comboBoxReverseMode.Items.Add("検出しない");
            comboBoxReverseMode.Items.Add("検出する");
            comboBoxReverseMode.SelectedIndex = 0;

            //サブピクセル推定
            comboBoxSubpxl.Items.Clear();
            comboBoxSubpxl.Items.Add("4近傍");
            comboBoxSubpxl.Items.Add("8近傍");
            comboBoxSubpxl.SelectedIndex = 0;

            //圧縮フィルタ
            comboBoxFilter.Items.Clear();
            comboBoxFilter.Items.Add("通常フィルタ");
            comboBoxFilter.Items.Add("再平均フィルタ");
            comboBoxFilter.Items.Add("最小値フィルタ");
            comboBoxFilter.Items.Add("最大値フィルタ");
            comboBoxFilter.SelectedIndex = 0;

            //実行オプション
            comboBoxExecuteOption.Items.Clear();
            comboBoxExecuteOption.Items.Add("Default");
            comboBoxExecuteOption.Items.Add("Enforce");
            comboBoxExecuteOption.Items.Add("Enforce2");
            comboBoxExecuteOption.SelectedIndex = 0;

            // 描画imageのセット
            cFviImageView1.Image = m_image;
            // オーバーレイの追加
            cFviImageView1.Display.Overlays.Add(m_overlay);
            // iniファイル選択
            fnSetIniFile();
        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            double msec = 0;

            try
            {
                m_overlay.Figures.Clear();
                formBroseIni.UseFileFlg = false;

                m_video.ChangeImageSize(m_image);

                //デバイスタイプ
                FVIL.Video.CFviContinuousGrab contgrab;
                switch (m_video.DeviceType)
                {
                    case FVIL.Video.DeviceType.FHC3312:
                        contgrab = ((FVIL.Video.CFviVideoFHC3312)m_video).ContinuousGrab;
                        ((FVIL.Video.CFviContinuousGrabFHC3312)contgrab).MemoryLock(m_image);
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        contgrab.GrabSync(1);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                    case FVIL.Video.DeviceType.FHC3313:
                        contgrab = ((FVIL.Video.CFviVideoFHC3313)m_video).ContinuousGrab;
                        ((FVIL.Video.CFviContinuousGrabFHC3313)contgrab).MemoryLock(m_image);
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        contgrab.GrabSync(1);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                    default:
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        m_video.GrabImageSync(m_image);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                }
                //測定時間表示
                TextBoxTime.Text = msec.ToString("0.000");

                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            // iniファイルの選択
            string strIniFilePath;

            strIniFilePath = "";
            if (formBroseIni.ShowDialog(this) == DialogResult.OK)
            {
                strIniFilePath = formBroseIni.strIniFilePath;
            }

            if (strIniFilePath.Length > 0)
            {
                try
                {
                    // 設定ファイルのロードとデバイスオープン
                    m_video = FVIL.Video.Function.CreateInstance(strIniFilePath);
                    // 取込画像を正規化
                    m_video.ChangeImageSize(m_image);
                    m_flgVideoOpened = true;

                    // ステータス表示の更新
                    // トリガモードの取得
                    ComboBoxTrigger.SelectedIndex = (int)m_video.Trigger;
                    // カメラ露光時間の取得
                    if (m_video.DeviceType != FVIL.Video.DeviceType.RICE001)
                    {
                        NumericUpDownExposure.Value = m_video.ExposureTime;
                    }
                    else
                    {
                        tabControlParam.TabPages.Remove(tabPageBoard);
                    }
                    ButtonGrabOne.Enabled = true;
                }
                catch (FVIL.CFviException ex)
                {
                    // エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    // エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else if (formBroseIni.UseFileFlg == true)
            {
                ButtonGrabOne.Enabled = false;
            }
            else
            {
                // iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }
        }

        /// <summary>
        /// LoadFile ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 入力画像ファイルのロード
        /// </remarks>
        private void buttonLoadImageFile_Click(object sender, EventArgs e)
        {
            
            //入力画像ファイルのロード
            openFileDialog1.FileName = null;
            openFileDialog1.Filter = "Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            openFileDialog1.Filter += "|Png ファイル (*.png)|*.png";
            openFileDialog1.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            openFileDialog1.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            openFileDialog1.DefaultExt = "*.bmp";

            if ((openFileDialog1.FileNames.Length <= 0) && (openFileDialog1.ShowDialog(this) == DialogResult.Cancel))
                return;

            try
            {
                FVIL.File.Function.LoadImageFile(openFileDialog1.FileName, m_image, FVIL.PixelMode.Unpacking);
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            formBroseIni.UseFileFlg = true;

            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// 処理範囲設定ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 処理範囲設定用フォーム表示。
        /// </remarks>
        private void buttonRegion_Click(object sender, EventArgs e)
        {
            //パタン表示フォーム
            Form_ProcDISPLAY formProcDisplay = new Form_ProcDISPLAY();

            //画像データ設定
            formProcDisplay.Set_image(m_image);
            formProcDisplay.Set_OverlayProc();

            //パタン表示画面
            formProcDisplay.ShowDialog(this);

            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// パタン登録ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パタン登録用フォーム表示
        /// </remarks>
        private void buttonSetPattern_Click(object sender, EventArgs e)
        {
            // パタン表示フォーム
            Form_ProcDISPLAY formProcDisplay = new Form_ProcDISPLAY();

            // パタンデータ設定
            formProcDisplay.Set_image(m_image);
            formProcDisplay.Set_pattern(m_pattern);
            formProcDisplay.Set_OverlayPattern();

            // パタン表示画面
            formProcDisplay.ShowDialog(this);

            // ボタンの有効化
            buttonDispPattern.Enabled = true;
            buttonSavePattern.Enabled = true;
            buttonDisplayMask.Enabled = true;

        }

        /// <summary>
        /// パタン読込ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パタン画像のロード
        /// </remarks>
        private void buttonLoadPattern_Click(object sender, EventArgs e)
        {
            //入力画像ファイルのロード
            openFileDialog1.FileName = null;
            openFileDialog1.Filter = "Pnt ファイル (*.pat)|*.pat";
            openFileDialog1.DefaultExt = "*.pat";
            if ((openFileDialog1.FileNames.Length <= 0) && (openFileDialog1.ShowDialog(this) == DialogResult.Cancel))
                return;

            try
            {
                m_pattern.Load(openFileDialog1.FileName);
                buttonDispPattern.Enabled = true;
                buttonSavePattern.Enabled = true;
                buttonDisplayMask.Enabled = true;
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// パタン表示ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パタン表示用フォーム表示。
        /// </remarks>
        private void buttonDispPattern_Click(object sender, EventArgs e)
        {
            //パタン表示フォーム
            Form_PTNDISPLAY formPtnDisplay = new Form_PTNDISPLAY();

            //パタンデータ設定
            formPtnDisplay.Set_pattern(m_pattern);

            //パタン表示フォーム表示変更
            formPtnDisplay.Text = "Pattern DISPLAY";
            formPtnDisplay.Mode_Pattern();

            //パタン表示画面
            formPtnDisplay.ShowDialog(this);

        }

        /// <summary>
        /// パタン保存ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パタン画像の保存
        /// </remarks>
        private void buttonSavePattern_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofdlg = new SaveFileDialog();
            ofdlg.FileName = "Pattern.pat";
            ofdlg.Filter = "Pat ファイル (*.pat)|*.pat";
            ofdlg.DefaultExt = "*.pat";
            ofdlg.OverwritePrompt = true;
            ofdlg.ShowDialog();

            if (ofdlg.FileNames.Length <= 0)
                return;

            try
            {
                //パタン画像の保存
                m_pattern.Save(ofdlg.FileName);
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// マスク設定ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// マスク設定用フォーム表示。
        /// </remarks>
        private void buttonDisplayMask_Click(object sender, EventArgs e)
        {
            //パタン表示フォーム
            Form_PTNDISPLAY formPtnDisplay = new Form_PTNDISPLAY();

            //パタンデータ設定
            formPtnDisplay.Set_pattern(m_pattern);

            //パタン表示フォーム表示変更
            formPtnDisplay.Text = "Pattern Mask DISPLAY";
            formPtnDisplay.Mode_Mask();

            //パタン表示画面
            formPtnDisplay.ShowDialog(this);
        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Active = true;
            else
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Active = false;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// x１ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります。

            //表示倍率の設定
            cFviImageView1.Display.Magnification = 1.0;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            cFviImageView1.FitImageSize();
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if (Value <= 0)
            {
                //0以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxTrigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.Trigger = (uint)ComboBoxTrigger.SelectedIndex;
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                    m_video.ExposureTime = (uint)NumericUpDownExposure.Value;
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);
                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch (ex.ErrorCode)
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// サーチ実行ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// サーチ処理・結果表示
        /// </remarks>
        private void buttonSearch_Click(object sender, EventArgs e)
        {
            // 表示初期化
            m_overlay.Figures.Clear();
            listViewResult.Items.Clear();
            //再描画
            cFviImageView1.Refresh();

            // 基本パラメータセット
            // --- ライブラリのオープン.
            m_gs2.Open();
            m_gs2.SrcImages[0] = m_image;

            // サーチ画像チェック
            if (FVIL.ErrorCode._SUCCESS != FVIL.GS2.CFviGS2.CheckValidity(m_image))
            {
                MessageBox.Show("入力画像が不正です", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // パラメータセット
            if (fnGS2ParamSet())
            {
                // サーチ実行
                if (fnGS2Exec())
                    // 結果表示
                    fnResultDisplay();
            }
        }

        /// <summary>
        /// パラメータ設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パラメータ設定関数
        /// </remarks>
        private bool fnGS2ParamSet()
        {
            bool rtn = false;

            try
            {
                // 件数
                m_gs2.Param.Count = (int)numericUpDownCount.Value;

                // サーチ精度
                switch (comboBoxPrecision.SelectedIndex)
                {
                    case 0: // 通常精度
                        m_gs2.Param.Precision = FVIL.GS2.Precision.Normal; break;
                    case 1: // 高精度
                        m_gs2.Param.Precision = FVIL.GS2.Precision.High; break;
                    case 2: // 超高精度
                        m_gs2.Param.Precision = FVIL.GS2.Precision.Super; break;
                    case 3: // ウルトラ超高精度
                        m_gs2.Param.Precision = FVIL.GS2.Precision.UltraSuper; break;
                }
                // 複雑度
                m_gs2.Param.Complexity = (int)numericUpDownComplexity.Value;

                // 途中相関値閾値
                m_gs2.Param.ThresholdMid = (int)numericUpDownThresholdMid.Value;
                // 最終相関値閾値
                m_gs2.Param.ThresholdFinal = (int)numericUpDownScoreThresholdFinal.Value;
                // 最小Ｘ間隔
                m_gs2.Param.PitchX = (int)numericUpDownPitchX.Value;
                // 最小Ｙ間隔
                m_gs2.Param.PitchY = (int)numericUpDownPitchY.Value;
                // サーチ開始圧縮度
                m_gs2.Param.FirstUnit = (int)numericUpDownFirstUnit.Value;
                // サーチ終了圧縮度
                m_gs2.Param.LastUnit = (int)numericUpDownLastUnit.Value;

                // 反転パタン検出
                switch (comboBoxReverseMode.SelectedIndex)
                {
                    //comboBoxReverseMode.Items.Add("検出しない");
                    //comboBoxReverseMode.Items.Add("検出する");
                    case 0:
                        m_gs2.Param.ReverseMode = false;  break;
                    case 1:
                        m_gs2.Param.ReverseMode = true;   break;
                }

                // サブピクセル推定
                switch (comboBoxSubpxl.SelectedIndex)
                {
                    //comboBoxSubpxl.Items.Add("4近傍");
                    //comboBoxSubpxl.Items.Add("8近傍");
                    case 0:
                        m_gs2.Param.Subpxl = FVIL.GS2.Subpxl.Neib4; break;
                    case 1:
                        m_gs2.Param.Subpxl = FVIL.GS2.Subpxl.Neib8; break;
                }
                // 処理範囲接触
                m_gs2.Param.EdgeDetect = checkEdgeDetect.Checked;

                //実行オプション
                switch (comboBoxExecuteOption.SelectedIndex)
                {
                    case 0: // 通常
                        m_gs2.ExecuteOption = FVIL.GS2.ExecuteOption.Default; break;
                    case 1: // 圧縮度強制
                        m_gs2.ExecuteOption = FVIL.GS2.ExecuteOption.Enforce; break;
                    case 2: // 圧縮度強制
                        m_gs2.ExecuteOption = FVIL.GS2.ExecuteOption.Enforce2; break;
                }
                

                // ---パタン関連----------------------------------------
                FVIL.GS2.Filter filter;
                switch (comboBoxFilter.SelectedIndex)
                {
                    //comboBoxFilter.Items.Add("通常フィルタ");
                    //comboBoxFilter.Items.Add("再平均フィルタ");
                    //comboBoxFilter.Items.Add("最小値フィルタ");
                    //comboBoxFilter.Items.Add("最大値フィルタ");
                    case 0:
                        filter = FVIL.GS2.Filter.Smooth; break;
                    case 1:
                        filter = FVIL.GS2.Filter.Avarage2; break;
                    case 2:
                        filter = FVIL.GS2.Filter.Min; break;
                    case 3:
                        filter = FVIL.GS2.Filter.Max; break;
                    default:
                        filter = FVIL.GS2.Filter.Smooth; break;
                }
                try
                {
                    // パタンの設定と作成
                    m_gs2.Pattern = m_gs2ptn;
                    m_gs2ptn.Create(m_pattern, filter);
                    rtn = true;
                }
                catch (FVIL.CFviException ex)
                {
                    // エラー処理
                    MessageBox.Show(String.Format("パタン作成に失敗しました。ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    rtn = false;
                }
                // ----------------------------------------------------


            }
            catch (FVIL.CFviException ex)
            {
                // エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                rtn = false;
            }
            catch (System.Exception ex)
            {
                // エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                rtn = false;
            }

            return rtn;
        }


        /// <summary>
        /// サーチ実行
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// グレイサーチ２実行関数
        /// </remarks>
        private bool fnGS2Exec()
        {
            bool rtn = false;

            try
            {
                // 画像処理実行
                m_timer.Start();
                m_gs2.Execute();
                msec = m_timer.Stop();
                rtn = true;
            }
            catch (FVIL.CFviException ex)
            {
                // エラー処理
                MessageBox.Show(String.Format("サーチエラーが発生しました。ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                rtn = false;
            }
            catch (System.Exception ex)
            {
                // エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                rtn = false;
            }
            return rtn;
        }

        /// <summary>
        /// 結果表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 結果表示関数
        /// </remarks>
        private void fnResultDisplay()
        {
            int i;

            // 描画クリア
            m_overlay.Figures.Clear();

            FVIL.Data.CFviPoint pat_mark = m_pattern.CenterMark;        // パタン登録時の基準点
            FVIL.Data.CFviRectangle pat_rect = m_pattern.RegistRect;    // パタン登録時の矩形

            // 結果のアドレス取得
            fvalgcli.F_GS_RESULT_PTR data_ptr = m_gs2.Result.Address;

            // 結果の表示
            for (i = 0; i < m_gs2.Result.Count; i++)
            {

                // サーチ結果の取得
                // --- 検出位置
                FVIL.Data.CFviPoint mark = new FVIL.Data.CFviPoint(data_ptr[0].x / 100, data_ptr[0].y / 100);

                // --- 検出位置とパタン情報から矩形を復元
                FVIL.Data.CFviRectangle rect = new FVIL.Data.CFviRectangle();
                rect.St = new FVIL.Data.CFviPoint(mark.X - pat_mark.X, mark.Y - pat_mark.Y);
                rect.Ed = new FVIL.Data.CFviPoint(rect.X + pat_rect.Width, rect.Y + pat_rect.Height);


                // 描画用.
                // --- 矩形(塗り潰し)
                FVIL.GDI.CFviGdiImage ol_fill = new FVIL.GDI.CFviGdiImage();
                ol_fill.SetSize(1, 1, 32);
                ol_fill.Position = rect.St;
                ol_fill.Alpha = 0x3F;
                ol_fill.StretchHorzSize = (int)(rect.Width);
                ol_fill.StretchVertSize = (int)(rect.Height);
                ol_fill.SetPixelRGB(0, 0, Color.FromArgb(0xFF, 0x00, 0x00, 0xFF));    // A,R,G,B

                // --- 矩形(枠)
                FVIL.GDI.CFviGdiRectangle ol_rect = new FVIL.GDI.CFviGdiRectangle(rect);
                ol_rect.Pen.Color = Color.FromArgb(0x00, 0x00, 0xFF);

                // --- 基準点
                FVIL.GDI.CFviGdiPoint ol_mark = new FVIL.GDI.CFviGdiPoint(mark);
                ol_mark.Size = new Size(5, 5);
                ol_mark.Pen.Color = Color.FromArgb(0xFF, 0x00, 0x00);

                // --- 相関値
                FVIL.GDI.CFviGdiString ol_cnt = new FVIL.GDI.CFviGdiString();
                ol_cnt.Text = i.ToString();
                ol_cnt.Position = mark;
                ol_cnt.Align = FVIL.GDI.TextAlign.Right | FVIL.GDI.TextAlign.Bottom;
                ol_cnt.BkMode = FVIL.GDI.BkMode.Transparent;
                ol_cnt.Color = Color.FromArgb(0xFF, 0xFF, 0x00);

                // 追加
                m_overlay.Figures.Add(ol_fill);
                m_overlay.Figures.Add(ol_rect);
                m_overlay.Figures.Add(ol_mark);
                m_overlay.Figures.Add(ol_cnt);

                //リスト表示
                if (i == 0)
                {
                    string[] listitem = { msec.ToString("0.000"), i.ToString(), (data_ptr[0].x / 100).ToString("0.00"), (data_ptr[0].y / 100).ToString("0.00"), data_ptr[0].score.ToString() };
                    listViewResult.Items.Add(new ListViewItem(listitem));
                }
                else
                {
                    string[] listitem = { "", i.ToString(), (data_ptr[0].x / 100).ToString("0.00"), (data_ptr[0].y / 100).ToString("0.00"), data_ptr[0].score.ToString() };
                    listViewResult.Items.Add(new ListViewItem(listitem));
                }

                // 次のデータへ・・
                data_ptr += 1;
            }
            //再描画
            cFviImageView1.Refresh();
        }


    }
}